//
//  HDProcessLister.m
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------

#import "HDProcessLister.h"


#include <assert.h>
#include <errno.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/sysctl.h>

typedef struct kinfo_proc kinfo_proc;


// thanks to <http://www.macosxguru.net/article.php?story=20060629075017862>
static int getProcessName( pid_t inPID, char *outName, size_t inMaxLen)
{
        struct kinfo_proc info;
        size_t length = sizeof(struct kinfo_proc);
        int mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, inPID };
                
        if (sysctl(mib, 4, &info, &length, NULL, 0) < 0)
                return -1 ;
        else
                strncpy(outName, info.kp_proc.p_comm, inMaxLen) ;

        return 0        ;
}


static int GetBSDProcessList(kinfo_proc **procList, size_t *procCount)
    // Returns a list of all BSD processes on the system.  This routine
    // allocates the list and puts it in *procList and a count of the
    // number of entries in *procCount.  You are responsible for freeing
    // this list (use "free" from System framework).
    // On success, the function returns 0.
    // On error, the function returns a BSD errno value.
{
    int                 err;
    kinfo_proc *        result;
    bool                done;
    static const int    name[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0 };
    // Declaring name as const requires us to cast it when passing it to
    // sysctl because the prototype doesn't include the const modifier.
    size_t              length;

    assert( procList != NULL);
    assert(*procList == NULL);
    assert(procCount != NULL);

    *procCount = 0;

    // We start by calling sysctl with result == NULL and length == 0.
    // That will succeed, and set length to the appropriate length.
    // We then allocate a buffer of that size and call sysctl again
    // with that buffer.  If that succeeds, we're done.  If that fails
    // with ENOMEM, we have to throw away our buffer and loop.  Note
    // that the loop causes use to call sysctl with NULL again; this
    // is necessary because the ENOMEM failure case sets length to
    // the amount of data returned, not the amount of data that
    // could have been returned.

    result = NULL;
    done = false;
    do {
        assert(result == NULL);

        // Call sysctl with a NULL buffer.

        length = 0;
        err = sysctl( (int *) name, (sizeof(name) / sizeof(*name)) - 1,
                      NULL, &length,
                      NULL, 0);
        if (err == -1) {
            err = errno;
        }

        // Allocate an appropriately sized buffer based on the results
        // from the previous call.

        if (err == 0) {
            result = malloc(length);
            if (result == NULL) {
                err = ENOMEM;
            }
        }

        // Call sysctl again with the new buffer.  If we get an ENOMEM
        // error, toss away our buffer and start again.

        if (err == 0) {
            err = sysctl( (int *) name, (sizeof(name) / sizeof(*name)) - 1,
                          result, &length,
                          NULL, 0);
            if (err == -1) {
                err = errno;
            }
            if (err == 0) {
                done = true;
            } else if (err == ENOMEM) {
                assert(result != NULL);
                free(result);
                result = NULL;
                err = 0;
            }
        }
    } while (err == 0 && ! done);

    // Clean up and establish post conditions.

    if (err != 0 && result != NULL) {
        free(result);
        result = NULL;
    }
    *procList = result;
    if (err == 0) {
        *procCount = length / sizeof(kinfo_proc);
    }

    assert( (err == 0) == (*procList != NULL) );

    return err;
}

@implementation HDProcessLister

const int MAX_PNAME_LEN = (2 << 5);

/*- (NSString *)nameOfProcessWithID:(pid_t pid) {
	NSString *processName = nil;
	
	char *pnam = NSZoneMalloc([self zone], sizeof(char) * MAX_PNAME_LEN);
	int r = getProcessName(pid, pnam, MAX_PNAME_LEN);
	
	if (r != -1) {
		//processName = [NSString stringWithCString:pnam encoding:NSISOLatin1StringEncoding];
		processName = [NSString stringWithUTF8String:pnam];
	}
	
	NSZoneFree([self zone], processName);
	
	return processName;
}*/

+ (BOOL)isProcessRunning:(NSString *)processName {
	BOOL processIsRunning = NO;

	const char *procName = [processName UTF8String];

	kinfo_proc *procList = NULL;
	size_t procCount = 0;
	
	int r1 = GetBSDProcessList(&procList, &procCount);
	if (r1 != -1) {
		int r2;
		char *runningProcName;
		
		int i;
		for (i = 0; i < procCount; ++i) {
			runningProcName = NSZoneMalloc([self zone], sizeof(char) * MAX_PNAME_LEN);
			r2 = getProcessName(procList[i].kp_proc.p_pid, runningProcName, MAX_PNAME_LEN);
			if (NULL != strstr(procName, runningProcName)) {
				processIsRunning = YES;
				NSZoneFree([self zone], runningProcName);
				break;
			}
			NSZoneFree([self zone], runningProcName);
		}
	}
	
	free(procList);
	
	return processIsRunning;

}

@end
